package cz.muni.fi.pa165.jpaexample;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

/**
 * Hello world!
 *
 */
public class App {

    public static String getCurrentNameInDb(EntityManagerFactory emf, Long id) {
        return emf.createEntityManager().find(Person.class, id).getName();
    }

    public static void main(String[] args) {

        EntityManagerFactory emf = Persistence.createEntityManagerFactory("TestPU");
        EntityManager em1 = emf.createEntityManager();
        EntityManager em2 = emf.createEntityManager();

        Person petr = new Person();
        petr.setFirstName("Petr");
        petr.setLastName("Adamek");

        Skill javaSkill = new Skill();
        javaSkill.setName("Java");
        javaSkill.setPerson(petr);
        petr.getSkills().add(javaSkill);
        
        
        em1.getTransaction().begin();
        em1.persist(petr);
        em1.getTransaction().commit();

        Long id = petr.getId();

        Person petr2 = em2.find(Person.class, id);

        em2.getTransaction().begin();
        petr2.setFirstName("Jan");
        em2.getTransaction().commit();

        em1.clear();
        
        Person petrX = em1.find(Person.class, id);
       
        em1.getTransaction().begin();
        petrX.setLastName("Novak");
        em1.getTransaction().rollback();

        System.err.println(petrX.getName());
        System.err.println(getCurrentNameInDb(emf, id));
        System.err.println(em1.contains(petrX));
        
        em1.getTransaction().begin();
        petrX = em1.find(Person.class, id);
        petrX.setLastName("Kvasnicka");
        em1.getTransaction().commit();

        System.err.println(petrX.getName());
        System.err.println(getCurrentNameInDb(emf, id));
        
        Query query = em1.createQuery(
                "SELECT p FROM Person p");
        for (Person p : (List<Person>) query.getResultList()) {
            System.err.println(p.getName());
        }

        TypedQuery<Person> query1 = em1.createQuery(
                "SELECT p FROM Person p", Person.class);
        for (Person p : query1.getResultList()) {
            System.err.println(p.getName());
        }

        TypedQuery<Person> query2 = em1.createNamedQuery(
                "findAllPersons", Person.class);
        for (Person p : query2.getResultList()) {
            System.err.println(p.getName());
        }

        TypedQuery<Person> query3 = em1.createQuery(
                "SELECT p FROM Person p WHERE p.firstName = 'Jan'", Person.class);
        for (Person p : query3.getResultList()) {
            System.err.println(p.getName());
        }

        TypedQuery<Long> query4 = em1.createQuery(
                "SELECT COUNT(p) FROM Person p", Long.class);
        System.err.println(query4.getSingleResult());

        TypedQuery<Object[]> query5 = em1.createQuery(
                "SELECT p.firstName,p.lastName,COUNT(s) "
                + "FROM Person p LEFT JOIN p.skills s "
                + "GROUP BY p.id,p.firstName,p.lastName", Object[].class);
        for (Object[] row : query5.getResultList()) {
            System.err.println(row[0] + " " + row[1] + ": " + row[2]);
        }

        TypedQuery<PersonTO> query6 = em1.createQuery(
                "SELECT new cz.muni.fi.pa165.jpaexample.PersonTO("
                + "p.firstName,p.lastName,COUNT(s)) "
                + "FROM Person p LEFT JOIN p.skills s "
                + "GROUP BY p.id,p.firstName,p.lastName", PersonTO.class);
        for (PersonTO personTO : query6.getResultList()) {
            System.err.println(personTO);
        }
        
        

    }
}
